# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/cifar10.ipynb.

# %% auto 0
__all__ = ['CIFAR10']

# %% ../nbs/cifar10.ipynb 3
from fastai.vision.all import *

import fastai_datasets.patches

# %% ../nbs/cifar10.ipynb 4
def CIFAR10():
    return DataBlock(
        blocks=(ImageBlock, CategoryBlock),
        get_items=get_image_files,
        get_y=parent_label,
        splitter=GrandparentSplitter('train', 'test'),
    ).datasets(untar_data(URLs.CIFAR))



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/Core/patches.ipynb.

# %% auto 0
__all__ = ['dl_defaults', 'ListToTuple']

# %% ../nbs/Core/patches.ipynb 2
import random
from collections import defaultdict
from typing import List, Dict, Sequence, Union
from functools import partial

from fastprogress.fastprogress import *
from fastai.vision.all import *

# %% ../nbs/Core/patches.ipynb 5
@patch
def sublist(self: TfmdLists, indices: Iterable[int]) -> TfmdLists:
    """a sublist that maintains laziness"""
    sub = self.new_empty()
    sub.items = [self.items[i] for i in indices]

    all_indices = L(range_of(self))
    def subsplit(s):
        split_idxs = set(all_indices[s])
        return [i for i, j in enumerate(indices) if j in split_idxs]
    sub.splits = [subsplit(s) for s in self.splits]
    
    return sub

# %% ../nbs/Core/patches.ipynb 9
@patch
def sub_dsets(self: Datasets, indices: Iterable[int]):
    return Datasets(tls=[t.sublist(indices) for t in self.tls])

# %% ../nbs/Core/patches.ipynb 13
@patch
def sub_dl(self:DataLoader, indices: Iterable[int]):
    return self.new(self.dataset.sub_dsets(indices))

# %% ../nbs/Core/patches.ipynb 18
@patch
def random_sub_dsets(self: Datasets, size, with_replacement=False, less_ok=False) -> Datasets:
    if size == 0:
        return self.subset([])
    if len(self) < size:
        assert less_ok
        size = len(self)
    sampler = random.choices if with_replacement else random.sample
    indices = sampler(range(len(self)),  k=size)
    return self.sub_dsets(indices)

# %% ../nbs/Core/patches.ipynb 21
@patch
@delegates(Datasets.random_sub_dsets)
def random_sub_dl(self: DataLoader, *args, **kwargs):
    return self.new(self.dataset.random_sub_dsets(*args, **kwargs))

# %% ../nbs/Core/patches.ipynb 25
@patch
def subset(self: TfmdLists, i):
    s = self._new(self._get(self.splits[i]), split_idx=i)
    s.splits = [slice(None), []]  # fastai bugfix
    return s

@patch
def __eq__(self: Union[Pipeline, Transform], other: Union[Pipeline, Transform]):
    """Needed to find shared transforms between TfmdLists"""
    return type(self) == type(other) and self.__dict__ == other.__dict__

@patch
def __add__(l1: TfmdLists, l2: TfmdLists):
    assert l1.split_idx == l2.split_idx

    tfms1, tfms2 = copy(list(l1.tfms)), copy(list(l2.tfms))
    merged_tfms = []
    while tfms1 and tfms2 and tfms1[-1] == tfms2[-1]:
        merged_tfms.insert(0, tfms1.pop())
        tfms2.pop()
    tfms1, tfms2 = Pipeline(tfms1), Pipeline(tfms2)

    return TfmdLists(
        [[i, item] for i, l in enumerate([l1, l2]) for item in l.items],
        tfms=[lambda o: [tfms1, tfms2][o[0]](o[1]), *merged_tfms],
        splits=[L(range_of(l1))[s1] + [i+len(l1) for i in L(range_of(l2))[s2]]
                for s1, s2 in zip_longest(l1.splits, l2.splits, fillvalue=[])],
        do_setup=False
    )

# %% ../nbs/Core/patches.ipynb 32
@patch
def __add__(self: Datasets, other: Datasets):
    assert len(self.tls) == len(other.tls)
    return Datasets(tls=[t1 + t2 for t1, t2 in zip(self.tls, other.tls)])

# %% ../nbs/Core/patches.ipynb 37
@patch
def __sub__(self: Datasets, other: Datasets):
    assert self.tfms == other.tfms
    other_items = set(other.items)
    assert other_items.issubset(self.items)
    return self.sub_dsets([i for i, o in enumerate(progress_bar(self.items, leave=False)) if o not in other_items])

# %% ../nbs/Core/patches.ipynb 40
from types import MethodType

@patch
def __add__(self: DataLoader, other: DataLoader):
    for k in list(self.__stored_args__.keys()) + self._methods:
        if k in {'dataset', 'n'}:
            continue
        v1 = getattr(self, k)
        v2 = getattr(other, k)
        if isinstance(v1, MethodType) and isinstance(v2, MethodType):
            continue
        assert v1 == v2, f"Property {k} mismatch: {v1} != {v2}"
    return self.new(self.dataset + other.dataset)

# %% ../nbs/Core/patches.ipynb 45
@patch(as_prop=True)
def i2t(self: Datasets):
    assert self.n_inp == len(self.tls) - 1
    return self.tls[-1]

# %% ../nbs/Core/patches.ipynb 47
@patch(as_prop=True)
def by_target(self: Datasets) -> Dict[int, Datasets]:
    if not hasattr(self, '_by_target'):
        targets = [int(t) for t in progress_bar(self.i2t, leave=False)]
        class_map = groupby(enumerate(targets), key=1, val=0)
        self._by_target = {self.vocab[c]: self.sub_dsets(indices)
                           for c, indices in progress_bar(class_map.items(), leave=False)}
    return self._by_target

# %% ../nbs/Core/patches.ipynb 50
@patch(as_prop=True)
def by_target(self: DataLoader) -> Dict[int, DataLoader]:
    return {k: self.new(v) for k, v in self.dataset.by_target.items()}

# %% ../nbs/Core/patches.ipynb 52
import matplotlib.pyplot as plt

@patch()
def plot_class_distribution(self: Datasets):
    for split in self.subsets:
        plt.bar(self.vocab, [len(split.by_target[c]) for c in self.vocab])

# %% ../nbs/Core/patches.ipynb 56
class ListToTuple(Transform):
    """Transforms lists to tuples, useful for fixing a bug in pytorch (pin_memory turns inner tuples into lists)"""
    def encodes(self, o:list):
        return tuple(o)

# %% ../nbs/Core/patches.ipynb 57
dl_defaults = {'pin_memory': default_device() != torch.device('cpu'), 'device': default_device(),
               'after_item': [ToTensor], 'after_batch': [ListToTuple, IntToFloatTensor]}

# %% ../nbs/Core/patches.ipynb 59
def _dl_args(**kwargs):
    kwargs = deepcopy(kwargs)
    args = deepcopy(dl_defaults)
    for event in ['after_item', 'after_batch']:
        if event in kwargs:
            tfms = kwargs.pop(event)
            args[event] += tfms if isinstance(tfms, Sequence) else [tfms]
    args.update(kwargs)
    return args

@patch
def dls(self: Datasets, **kwargs) -> DataLoaders:
    """Calls `Datasets.dataloaders` with defaults from `dl_defaults`"""
    return self.dataloaders(**_dl_args(**kwargs))

@patch
def dl(self: Datasets, **kwargs) -> DataLoader:
    """Creates a `DataLoader` (ignoring splits) with defaults from `dl_defaults`"""
    return self._dl_type(self, **_dl_args(**kwargs))

# %% ../nbs/Core/patches.ipynb 61
@patch
def load(self: Datasets, **kwargs):
    return first(self.dl(bs=len(self), **kwargs))

# %% ../nbs/Core/patches.ipynb 64
@patch(as_prop=True)
def subsets(self: Datasets) -> TfmdLists:
    """Lazy list of a `Datasets`'s subsets"""
    return TfmdLists(range(self.n_subsets), self.subset)

# %% ../nbs/Core/patches.ipynb 66
@patch
def resplit(self: Datasets,
            splits: Union[Callable, List[List[int]]]  # a splitter function or a list of splits
            ):
    """Sets the splits of a `Datasets`"""
    if isinstance(splits, Callable):
        splits = splits(self)
    for t in self.tls:
        t.splits = splits

# %% ../nbs/Core/patches.ipynb 69
@patch()
def __repr__(self: Datasets):
    return '['+'\n'.join(repr(s) for s in self.subsets)+']' if self.split_idx is None else coll_repr(self)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/pairs.ipynb.

# %% auto 0
__all__ = ['ImagePair', 'Sameness', 'Pairs']

# %% ../nbs/pairs.ipynb 2
import random
from typing import Union

from fastai.vision.all import *
from fastprogress.fastprogress import *

import fastai_datasets.patches
from .utils import *

# %% ../nbs/pairs.ipynb 3
class ImagePair(fastuple):
    """Adds showing functionality to fastai's `fastuple`"""
    @classmethod
    def create(cls, fns): return cls(tuple(PILImage.create(f) for f in fns))
    
    def show(self, ctx=None, **kwargs):
        t1,t2 = ToTensor()(self)
        assert type(t1) == type(t2)
        line = t1.new_zeros((t1.shape[0], t1.shape[1], 1))
        joint_image = torch.cat([t1,line,t2], dim=2).as_subclass(type(t1))
        return joint_image.show(ctx=ctx, **kwargs)

class Sameness(Categorize):
    """Converts boolean/binary labels into "Not Same"/"Same" labels"""
    def __init__(self):
        super().__init__(['Not Same', 'Same'], sort=False)

    def encodes(self, o: Union[bool, int]):
        return super().encodes(self.vocab[o])

# %% ../nbs/pairs.ipynb 5
@typedispatch
def show_batch(x:ImagePair, y, samples, ctxs=None, max_n=9, nrows=None, ncols=3, figsize=None, **kwargs):
    if figsize is None: figsize = (ncols*4, max_n//ncols * 2)
    if ctxs is None: ctxs = get_grid(min(len(samples), max_n), nrows=nrows, ncols=ncols, figsize=figsize)
    ctxs = show_batch[TensorImage](x, y, samples, ctxs=ctxs, max_n=max_n, **kwargs)
    return ctxs

@typedispatch
def show_results(x:ImagePair, y:TensorCategory, samples, outs, ctxs=None, max_n=9, nrows=None, ncols=3, figsize=None, **kwargs):
    if figsize is None: figsize = (ncols*4, max_n//ncols * 2.5)
    if ctxs is None: ctxs = get_grid(min(len(samples), max_n), nrows=nrows, ncols=ncols, figsize=figsize)
    ctxs = show_results[TensorImage, TensorCategory](x, y, samples, outs, ctxs=ctxs, max_n=max_n, **kwargs)

# %% ../nbs/pairs.ipynb 6
def _pairs_for_split(singles: DataLoaders, split_idx: int, factor: int):
    assert singles.n_inp == 1

    indices = L(range_of(singles))[singles.splits[split_idx]]
    num = int(len(indices) * factor)

    class_map = defaultdict(list)
    for i, c in progress_bar(indices.zipwith(singles.i2t.subset(split_idx)), leave=False):
        class_map[singles.vocab[c]].append(i)

    @return_list
    def _positive_pairs():
        multi_item_class_map = {k: v for k, v in class_map.items() if len(v)>1}
        for _ in progress_bar(range(num//2), leave=False):
            c, idxs = random.choice(list(multi_item_class_map.items()))
            yield tuple(random.sample(idxs, 2))

    @return_list
    def _negative_pairs():
        for _ in progress_bar(range(num//2), leave=False):
            (c1, idxs1), (c2, idxs2) = random.sample(list(class_map.items()), 2)
            yield (random.choice(idxs1), random.choice(idxs2))

    return _positive_pairs() + _negative_pairs()


def Pairs(singles: Datasets,  # Used to construct pairs
          factor=10,  # ratio between numer of pairs and number of single items, for each split
          tuple_type=ImagePair  # To add `show` logic for pairs
          ) -> Datasets:
    """Fixed dataset of randomly-sampled pairs """
    assert singles.n_inp == 1
    pairs_for_splits = [_pairs_for_split(singles, i, factor) for i in range(singles.n_subsets)]

    splits = [slice(len(pairs_for_splits[0]))]
    for pairs in pairs_for_splits[1:]:
        start = splits[-1].stop
        splits.append(slice(start, start+len(pairs)))

    return Datasets(
        [L(x) for items in pairs_for_splits for x in items],
        tfms=[
            lambda o: tuple_type(singles.tls[0][o]),
            [lambda o: bool(singles.i2t[o[0]] == singles.i2t[o[1]]), Sameness()]
        ],
        splits=splits
    )



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/Core/utils.ipynb.

# %% auto 0
__all__ = ['data_path', 'fetch_file', 'return_list', 'mtcnn_aligned']

# %% ../nbs/Core/utils.ipynb 3
from fastcore.all import *
from fastdownload import *
from fastai.vision.all import *

# %% ../nbs/Core/utils.ipynb 4
def data_path():
    return fastai_path('data')

@delegates(FastDownload.download)
def fetch_file(*args, **kwargs):
    return FastDownload(fastai_cfg()).download(*args, **kwargs)

# %% ../nbs/Core/utils.ipynb 5
def return_list(f):
    @delegates(f)
    def wrapper(*args, **kwargs):
        return list(f(*args, **kwargs))
    return wrapper

# %% ../nbs/Core/utils.ipynb 7
"""
based on https://github.com/timesler/facenet-pytorch/blob/master/examples/lfw_evaluate.ipynb
"""
import os

import PIL
import torch
from facenet_pytorch import MTCNN, training
from fastprogress.fastprogress import *


def mtcnn_aligned(path: Path,  # path to unaligned images
                  force=False,  # compute MTCNN alignment even if aligned images exist
                  batched=True
                  ) -> Path:   # path to aligned images
    """Uses MTCNN to align and extract faces"""
    mtcnn_path = path.with_name(path.name+'_mtcnn')
    if not force and mtcnn_path.exists():
        return mtcnn_path

    mtcnn = MTCNN(
        image_size=160,
        margin=14,
        device=default_device(),
        selection_method='center_weighted_size'
    )

    loader = torch.utils.data.DataLoader(
        Datasets(get_image_files(path), [PIL.Image.open, noop]),
        num_workers=0 if os.name == 'nt' else 8,
        batch_size=64 if batched else 1,
        collate_fn=training.collate_pil
    )

    for imgs, paths in progress_bar(loader):
        output_paths = [mtcnn_path/p.relative_to(path) for p in paths]
        mtcnn(imgs, save_path=output_paths)

    return mtcnn_path



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/imagenette.ipynb.

# %% auto 0
__all__ = ['Imagenette']

# %% ../nbs/imagenette.ipynb 3
from typing import Literal

from fastai.vision.all import *

import fastai_datasets.patches

# %% ../nbs/imagenette.ipynb 4
_labels = dict(
    n01440764='tench',
    n02102040='English springer',
    n02979186='cassette player',
    n03000684='chain saw',
    n03028079='church',
    n03394916='French horn',
    n03417042='garbage truck',
    n03425413='gas pump',
    n03445777='golf ball',
    n03888257='parachute'
)

def Imagenette(size:Literal[None, 320, 160]):
    url = {None: URLs.IMAGENETTE, 320: URLs.IMAGENETTE_320, 160: URLs.IMAGENETTE_160}[size]
    return DataBlock(
        blocks=(ImageBlock, CategoryBlock),
        get_items=get_image_files,
        get_y=Pipeline([parent_label, _labels.__getitem__]),
        splitter=GrandparentSplitter('train', 'val')
    ).datasets(untar_data(url))



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/mnist.ipynb.

# %% auto 0
__all__ = ['MNIST', 'TinyMNIST']

# %% ../nbs/mnist.ipynb 3
from functools import partial

from fastai.vision.all import *

import fastai_datasets.patches

# %% ../nbs/mnist.ipynb 4
def MNIST(sample=False):
    return DataBlock(
        blocks=(ImageBlock(PILImageBW), CategoryBlock),
        get_items=get_image_files,
        get_y=parent_label,
        splitter=GrandparentSplitter('training', 'testing') if not sample else GrandparentSplitter()
    ).datasets(untar_data(URLs.MNIST if not sample else URLs.MNIST_SAMPLE))

def TinyMNIST():
    return DataBlock(
        blocks=(ImageBlock(PILImageBW), CategoryBlock),
        get_items=partial(get_image_files, folders=['train', 'valid']),
        get_y=parent_label,
        splitter=GrandparentSplitter(),
    ).datasets(untar_data(URLs.MNIST_TINY))



__version__ = "0.0.7"

# Bugfix: fastprogress bars not showing in VSCode notebooks. Taken from https://github.com/microsoft/vscode-jupyter/issues/13163
import IPython
def update_patch(self, obj):
    IPython.display.clear_output(wait=True)
    self.display(obj)
IPython.display.DisplayHandle.update = update_patch


# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/fastai-datasets',
                'doc_host': 'https://Irad-Zehavi.github.io',
                'git_url': 'https://github.com/Irad-Zehavi/fastai-datasets',
                'lib_path': 'fastai_datasets'},
  'syms': { 'fastai_datasets.all': {},
            'fastai_datasets.cifar10': {'fastai_datasets.cifar10.CIFAR10': ('cifar10.html#cifar10', 'fastai_datasets/cifar10.py')},
            'fastai_datasets.imagenette': { 'fastai_datasets.imagenette.Imagenette': ( 'imagenette.html#imagenette',
                                                                                       'fastai_datasets/imagenette.py')},
            'fastai_datasets.lfw': { 'fastai_datasets.lfw.LFW': ('Facial Recognition/lfw.html#lfw', 'fastai_datasets/lfw.py'),
                                     'fastai_datasets.lfw.LFW.__init__': ( 'Facial Recognition/lfw.html#lfw.__init__',
                                                                           'fastai_datasets/lfw.py'),
                                     'fastai_datasets.lfw.LFW._fetch_file': ( 'Facial Recognition/lfw.html#lfw._fetch_file',
                                                                              'fastai_datasets/lfw.py'),
                                     'fastai_datasets.lfw.LFW._get_path': ( 'Facial Recognition/lfw.html#lfw._get_path',
                                                                            'fastai_datasets/lfw.py'),
                                     'fastai_datasets.lfw.LFW._load': ('Facial Recognition/lfw.html#lfw._load', 'fastai_datasets/lfw.py'),
                                     'fastai_datasets.lfw.LFW._parse_items': ( 'Facial Recognition/lfw.html#lfw._parse_items',
                                                                               'fastai_datasets/lfw.py'),
                                     'fastai_datasets.lfw.LFW._url': ('Facial Recognition/lfw.html#lfw._url', 'fastai_datasets/lfw.py'),
                                     'fastai_datasets.lfw.LFW.test': ('Facial Recognition/lfw.html#lfw.test', 'fastai_datasets/lfw.py'),
                                     'fastai_datasets.lfw.LFWDevMixin': ( 'Facial Recognition/lfw.html#lfwdevmixin',
                                                                          'fastai_datasets/lfw.py'),
                                     'fastai_datasets.lfw.LFWDevMixin.dev': ( 'Facial Recognition/lfw.html#lfwdevmixin.dev',
                                                                              'fastai_datasets/lfw.py'),
                                     'fastai_datasets.lfw.LFWPairs': ('Facial Recognition/lfw.html#lfwpairs', 'fastai_datasets/lfw.py'),
                                     'fastai_datasets.lfw.LFWPairsMixin': ( 'Facial Recognition/lfw.html#lfwpairsmixin',
                                                                            'fastai_datasets/lfw.py'),
                                     'fastai_datasets.lfw.LFWPairsMixin._load': ( 'Facial Recognition/lfw.html#lfwpairsmixin._load',
                                                                                  'fastai_datasets/lfw.py'),
                                     'fastai_datasets.lfw.LFWPairsMixin._parse_items': ( 'Facial '
                                                                                         'Recognition/lfw.html#lfwpairsmixin._parse_items',
                                                                                         'fastai_datasets/lfw.py'),
                                     'fastai_datasets.lfw.LFWPeople': ('Facial Recognition/lfw.html#lfwpeople', 'fastai_datasets/lfw.py'),
                                     'fastai_datasets.lfw.LFWPeople._load': ( 'Facial Recognition/lfw.html#lfwpeople._load',
                                                                              'fastai_datasets/lfw.py'),
                                     'fastai_datasets.lfw.LFWPeople._parse_items': ( 'Facial Recognition/lfw.html#lfwpeople._parse_items',
                                                                                     'fastai_datasets/lfw.py'),
                                     'fastai_datasets.lfw.SLLFWPairs': ('Facial Recognition/lfw.html#sllfwpairs', 'fastai_datasets/lfw.py'),
                                     'fastai_datasets.lfw.SLLFWPairs._parse_items': ( 'Facial Recognition/lfw.html#sllfwpairs._parse_items',
                                                                                      'fastai_datasets/lfw.py')},
            'fastai_datasets.mnist': { 'fastai_datasets.mnist.MNIST': ('mnist.html#mnist', 'fastai_datasets/mnist.py'),
                                       'fastai_datasets.mnist.TinyMNIST': ('mnist.html#tinymnist', 'fastai_datasets/mnist.py')},
            'fastai_datasets.pairs': { 'fastai_datasets.pairs.ImagePair': ('pairs.html#imagepair', 'fastai_datasets/pairs.py'),
                                       'fastai_datasets.pairs.ImagePair.create': ( 'pairs.html#imagepair.create',
                                                                                   'fastai_datasets/pairs.py'),
                                       'fastai_datasets.pairs.ImagePair.show': ('pairs.html#imagepair.show', 'fastai_datasets/pairs.py'),
                                       'fastai_datasets.pairs.Pairs': ('pairs.html#pairs', 'fastai_datasets/pairs.py'),
                                       'fastai_datasets.pairs.Sameness': ('pairs.html#sameness', 'fastai_datasets/pairs.py'),
                                       'fastai_datasets.pairs.Sameness.__init__': ( 'pairs.html#sameness.__init__',
                                                                                    'fastai_datasets/pairs.py'),
                                       'fastai_datasets.pairs.Sameness.encodes': ( 'pairs.html#sameness.encodes',
                                                                                   'fastai_datasets/pairs.py'),
                                       'fastai_datasets.pairs._pairs_for_split': ( 'pairs.html#_pairs_for_split',
                                                                                   'fastai_datasets/pairs.py'),
                                       'fastai_datasets.pairs.show_batch': ('pairs.html#show_batch', 'fastai_datasets/pairs.py'),
                                       'fastai_datasets.pairs.show_results': ('pairs.html#show_results', 'fastai_datasets/pairs.py')},
            'fastai_datasets.patches': { 'fastai_datasets.patches.DataLoader.__add__': ( 'Core/patches.html#dataloader.__add__',
                                                                                         'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.DataLoader.by_target': ( 'Core/patches.html#dataloader.by_target',
                                                                                           'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.DataLoader.random_sub_dl': ( 'Core/patches.html#dataloader.random_sub_dl',
                                                                                               'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.DataLoader.sub_dl': ( 'Core/patches.html#dataloader.sub_dl',
                                                                                        'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.Datasets.__add__': ( 'Core/patches.html#datasets.__add__',
                                                                                       'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.Datasets.__repr__': ( 'Core/patches.html#datasets.__repr__',
                                                                                        'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.Datasets.__sub__': ( 'Core/patches.html#datasets.__sub__',
                                                                                       'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.Datasets.by_target': ( 'Core/patches.html#datasets.by_target',
                                                                                         'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.Datasets.dl': ( 'Core/patches.html#datasets.dl',
                                                                                  'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.Datasets.dls': ( 'Core/patches.html#datasets.dls',
                                                                                   'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.Datasets.i2t': ( 'Core/patches.html#datasets.i2t',
                                                                                   'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.Datasets.load': ( 'Core/patches.html#datasets.load',
                                                                                    'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.Datasets.plot_class_distribution': ( 'Core/patches.html#datasets.plot_class_distribution',
                                                                                                       'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.Datasets.random_sub_dsets': ( 'Core/patches.html#datasets.random_sub_dsets',
                                                                                                'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.Datasets.resplit': ( 'Core/patches.html#datasets.resplit',
                                                                                       'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.Datasets.sub_dsets': ( 'Core/patches.html#datasets.sub_dsets',
                                                                                         'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.Datasets.subsets': ( 'Core/patches.html#datasets.subsets',
                                                                                       'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.ListToTuple': ( 'Core/patches.html#listtotuple',
                                                                                  'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.ListToTuple.encodes': ( 'Core/patches.html#listtotuple.encodes',
                                                                                          'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.TfmdLists.__add__': ( 'Core/patches.html#tfmdlists.__add__',
                                                                                        'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.TfmdLists.sublist': ( 'Core/patches.html#tfmdlists.sublist',
                                                                                        'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.TfmdLists.subset': ( 'Core/patches.html#tfmdlists.subset',
                                                                                       'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches.Union[(Pipeline, Transform)].__eq__': ( 'Core/patches.html#union[(pipeline, '
                                                                                                          'transform)].__eq__',
                                                                                                          'fastai_datasets/patches.py'),
                                         'fastai_datasets.patches._dl_args': ('Core/patches.html#_dl_args', 'fastai_datasets/patches.py')},
            'fastai_datasets.pfr': { 'fastai_datasets.pfr.PinterestFaces': ( 'Facial Recognition/pfr.html#pinterestfaces',
                                                                             'fastai_datasets/pfr.py')},
            'fastai_datasets.utils': { 'fastai_datasets.utils.data_path': ('Core/utils.html#data_path', 'fastai_datasets/utils.py'),
                                       'fastai_datasets.utils.fetch_file': ('Core/utils.html#fetch_file', 'fastai_datasets/utils.py'),
                                       'fastai_datasets.utils.mtcnn_aligned': ('Core/utils.html#mtcnn_aligned', 'fastai_datasets/utils.py'),
                                       'fastai_datasets.utils.return_list': ('Core/utils.html#return_list', 'fastai_datasets/utils.py')}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/Facial Recognition/pfr.ipynb.

# %% auto 0
__all__ = ['ROOT', 'PinterestFaces']

# %% ../nbs/Facial Recognition/pfr.ipynb 4
from fastai.vision.all import *

import fastai_datasets.patches
from .utils import *

# %% ../nbs/Facial Recognition/pfr.ipynb 5
ROOT = data_path()/'105_classes_pins_dataset'

def PinterestFaces(mtcnn=True):
    """Requires the user to download the file manually"""
    assert ROOT.exists(), f'Please manually download the dataset to {ROOT}'
    dblock = DataBlock(
        blocks=(ImageBlock, CategoryBlock),
        get_items=get_image_files,
        get_y=lambda p: parent_label(p).replace('pins_', '').replace('_', ' ').title()
    )

    return dblock.datasets(mtcnn_aligned(ROOT, batched=False) if mtcnn else ROOT)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/Facial Recognition/lfw.ipynb.

# %% auto 0
__all__ = ['LFWPeople', 'LFWPairs', 'SLLFWPairs']

# %% ../nbs/Facial Recognition/lfw.ipynb 2
from abc import ABC, abstractmethod

from fastai.vision.all import *
from sklearn.model_selection import KFold

import fastai_datasets.patches
from .utils import *

# %% ../nbs/Facial Recognition/lfw.ipynb 3
class LFW(ABC):
    BASE_URL = 'http://vis-www.cs.umass.edu/lfw'
    TEST_ITEMS_FILE_NAME: str

    def __init__(self, mtcnn=True):
        self.root = untar_data(self._url('lfw.tgz'))
        if mtcnn:
            self.root = mtcnn_aligned(self.root)


    @classmethod
    def _url(cls, fname):
        return f'{cls.BASE_URL}/{fname}'

    def test(self):
        items = self._parse_items(self.TEST_ITEMS_FILE_NAME)
        splits = KFold(n_splits=10, shuffle=False).split(range_of(items))
        return [self._load(items=items, splits=s) for s in splits]

    def _fetch_file(self, fname):
        return fetch_file(self._url(fname))

    @abstractmethod
    def _parse_items(self, fname):
        pass

    @abstractmethod
    def _load(self, **kwargs):
        pass

    def _get_path(self, name, num) -> Path:
        return self.root / name / f'{name}_{num:04d}.jpg'


class LFWDevMixin(LFW):
    DEV_TRAIN_ITEMS_FILE_NAME: str
    DEV_TEST_ITEMS_FILE_NAME: str

    def dev(self):
        train_items = self._parse_items(self.DEV_TRAIN_ITEMS_FILE_NAME)
        valid_items = self._parse_items(self.DEV_TEST_ITEMS_FILE_NAME)
        items = valid_items+train_items

        return self._load(
            items=items,
            splits=IndexSplitter(range_of(valid_items))(items)
        )

# %% ../nbs/Facial Recognition/lfw.ipynb 4
class LFWPeople(LFWDevMixin, LFW):
    """
    Individual facial images.
    Splits contain disjoint identities, since they're meant to for constructing pairs (using `Pairs`)
    """
    TEST_ITEMS_FILE_NAME = 'people.txt'
    DEV_TRAIN_ITEMS_FILE_NAME = 'peopleDevTrain.txt'
    DEV_TEST_ITEMS_FILE_NAME = 'peopleDevTest.txt'
    
    @return_list
    def _parse_items(self, fname):
        lines = [l.split() for l in self._fetch_file(fname).readlines()]
        for l in lines[1:]:
            if len(l) == 1:
                continue
            name, num_images = l
            for i in range(1, int(num_images)+1):
                yield self._get_path(name, i)

    def _load(self, **kwargs):
        return Datasets(
            tfms=[
                PILImage.create,
                [parent_label, lambda s: s.replace('_', ' '), Categorize()]
            ],
            train_setup=False,
            **kwargs
        )


# %% ../nbs/Facial Recognition/lfw.ipynb 6
from .pairs import *

# %% ../nbs/Facial Recognition/lfw.ipynb 7
class LFWPairsMixin(LFW):
    """Fixed pairs of facial images"""
    TEST_ITEMS_FILE_NAME = 'pairs.txt'
    DEV_TRAIN_ITEMS_FILE_NAME = 'pairsDevTrain.txt'
    DEV_TEST_ITEMS_FILE_NAME = 'pairsDevTest.txt'

    @return_list
    def _parse_items(self, fname):
        lines = self._fetch_file(fname).readlines()
        for l in lines[1:]:
            l = l.split()
            if len(l) == 3:
                name, num1, num2 = l
                yield [self._get_path(name, int(num1)),
                       self._get_path(name, int(num2))]
            else:
                name1, num1, name2, num2 = l
                yield [self._get_path(name1, int(num1)),
                       self._get_path(name2, int(num2))]
    
    def _load(self, **kwargs):
        return Datasets(
            tfms=[
                ImagePair.create,
                [lambda pair: parent_label(pair[0])==parent_label(pair[1]), Sameness()]
            ],
            train_setup=False,
            **kwargs
        )

# %% ../nbs/Facial Recognition/lfw.ipynb 8
class LFWPairs(LFWDevMixin, LFWPairsMixin, LFW):
    pass

# %% ../nbs/Facial Recognition/lfw.ipynb 10
class SLLFWPairs(LFWPairsMixin, LFW):
    """Similar Looking LFW: http://whdeng.cn/SLLFW/index.html"""
    BASE_URL = f'http://whdeng.cn/SLLFW'
    TEST_ITEMS_FILE_NAME = 'pair_SLLFW.txt'

    @return_list
    def _parse_items(self, fname):
        # Parsed according to http://www.whdeng.cn/SLLFW/index.html#download
        singles = re.findall(r'(.*)/.*_(\d*)', self._fetch_file(fname).read_text())
        pairs = [(singles[i], singles[i+1]) for i in range(0, len(singles), 2)]
        
        for ((name1, num1), (name2, num2)) in pairs:
            yield [self._get_path(name1, int(num1)),
                   self._get_path(name2, int(num2))]
